"use strict";
/* eslint-disable prefer-rest-params */
/* eslint-disable @typescript-eslint/ban-types */
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const path_1 = __importDefault(require("path"));
const fs_1 = __importDefault(require("fs"));
const yeoman_generator_1 = __importDefault(require("yeoman-generator"));
const fast_glob_1 = __importDefault(require("fast-glob"));
const debug_1 = __importDefault(require("debug"));
const debug = (0, debug_1.default)('yo:magicdawn:docs');
class DocsGenerator extends yeoman_generator_1.default {
    constructor(args, opts) {
        debug('constructor arguments %j', arguments);
        super(args, opts);
    }
    /**
     * we starts here
     */
    async default() {
        this._addDocs();
    }
    _addDocs() {
        this.sourceRoot(__dirname + '/docs');
        const destDirName = path_1.default.basename(this.destinationRoot());
        let pkg;
        if (fs_1.default.existsSync(this.destinationPath('package.json'))) {
            pkg = this.fs.readJSON(this.destinationPath('package.json'));
        }
        const data = {
            name: pkg.name || destDirName,
            description: pkg.description || 'An awesome project',
        };
        // website
        const files = fast_glob_1.default.sync([
            '!**/node_modules',
            // website
            'website/**/*.*',
            '!website/translated_docs',
            '!website/build/',
            '!website/yarn.lock',
            '!website/node_modules',
            '!website/i18n/',
            // md
            'md/*.md',
            '!md/changelog.md',
            // .git
            '.gitignore',
            '!gitignore',
        ], {
            cwd: this.sourceRoot(),
        });
        // 要使用 copyTpl
        const useTplFiles = ['md/index.md', 'website/siteConfig.js'];
        for (const f of files) {
            if (useTplFiles.includes(f)) {
                this.fs.copyTpl(this.templatePath(f), this.destinationPath('docs/' + f), data);
            }
            else {
                this.fs.copy(this.templatePath(f), this.destinationPath('docs/' + f));
            }
        }
        // npm 发布会去除 .gitignore
        if (!fs_1.default.existsSync(this.templatePath('.gitignore'))) {
            this.fs.copy(this.templatePath('gitignore'), this.destinationPath('docs/.gitignore'));
        }
    }
}
exports.default = DocsGenerator;
