"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
/* eslint-disable @typescript-eslint/ban-types */
const debug_1 = __importDefault(require("debug"));
const fs_extra_1 = __importDefault(require("fs-extra"));
const yeoman_generator_1 = __importDefault(require("yeoman-generator"));
const debug = (0, debug_1.default)('yo:magicdawn:dot-files');
const OPTIONS = [
    // eslint
    {
        name: '.eslintrc.yml',
        desc: 'use @magicdawn/eslint-config',
    },
    // prettier
    {
        name: 'prettier.config.js',
        desc: 'use @magicdawn/prettier-config',
    },
    // ts
    {
        name: 'tsconfig.json',
        desc: 'TypeScript config',
    },
    // .github
    {
        name: '.github/workflows/ci.yml',
        desc: 'GitHub Actions config',
    },
];
class default_1 extends yeoman_generator_1.default {
    constructor(...args) {
        super(...args);
        // project root
        this.sourceRoot(__dirname + '/../../');
    }
    async default() {
        const { dotfiles } = await this._prompt();
        this._copyFiles(dotfiles);
    }
    async _prompt() {
        const answers = await this.prompt([
            {
                type: 'checkbox',
                message: '选择 dot-files',
                name: 'dotfiles',
                choices: [
                    ...OPTIONS.map((item) => {
                        return { name: `${item.name} (${item.desc})`, value: item.name };
                    }),
                ],
            },
        ]);
        return answers;
    }
    _getDotFilePath(name) {
        let file;
        const tryFile = () => {
            // this.fs.exists 不能判断 .github 文件夹
            if (file && fs_extra_1.default.existsSync(file)) {
                return file;
            }
            else {
                debug('not exists: %s', file);
            }
        };
        // patch .gitignore
        if (name === '.gitignore') {
            file = this.templatePath('templates/app/' + name.slice(1));
            if (tryFile())
                return file;
        }
        file = this.templatePath('templates/app/' + name);
        if (tryFile())
            return file;
        file = this.templatePath('templates/dotfiles/' + name);
        if (tryFile())
            return file;
        throw new Error(`can not find dotfile name = ${name}`);
    }
    async _copyFiles(dotfiles) {
        for (const item of dotfiles) {
            const src = this._getDotFilePath(item);
            debug('resolved %s -> %s', item, src);
            const dest = this.destinationPath(item);
            this.fs.copy(src, dest);
        }
    }
}
exports.default = default_1;
